#include <fstream.h>
#include <io.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <string.h>
#include <conio.h>
#include <dos.h>
#include <dir.h>
#include <errno.h>
#include <stdlib.h>
#include <alloc.h>

#include "flexlist.hpp"

#define __DEBUG__

static const MAXIOBUF = 1024;
static const char * PNAME = "MovPkt";
static const char * PAUTH = "J.R.Louvau";
static const char * PCOPY = "1991";
static const unsigned PVERS = 0x0100;

#ifdef __DEBUG__
static const unsigned BVERS = 1;
#endif

void copypkt(char * oname, char * iname)
{
   char    ibuf[MAXIOBUF],
           obuf[MAXIOBUF],
           buff[MAXIOBUF];
   fstream Ifile,
           Ofile;

   Ifile.setbuf(ibuf,MAXIOBUF);
   Ofile.setbuf(obuf,MAXIOBUF);

   Ifile.open(iname,ios::in|ios::binary);
   if (!Ifile)
   {
      cprintf("   %s\r\n",strerror(errno));
      return;
   }

   Ofile.open(oname,ios::out|ios::binary);
   if (!Ofile)
   {
      cprintf("   %s\r\n",strerror(errno));
      Ifile.close();
      return;
   }

   while ((!Ifile.read(buff,MAXIOBUF).eof()) && (!Ofile.bad()))
   {
      Ofile.write(buff,Ifile.gcount());
   }

   if (Ifile.bad() || Ofile.bad())
   {
      cprintf("   %s\r\n",strerror(errno));
      Ifile.close();
      Ofile.close();
      unlink(oname);
      return;
   }

   Ifile.close();
   Ofile.close();
   unlink(iname);
}

void movepkt(char * oname, char * iname)
{
   if (rename(iname,oname) == -1)
   {
      switch (errno)
      {
         case ENOTSAM:
         case ENOENT:
            copypkt(oname,iname);
            break;
         default:
            cprintf("   %s\r\n",strerror(errno));
            break;
      }
   }
}

void usage()
{
   textattr(LIGHTGRAY);
   cputs("Usage:\r\n\n");
   cprintf("   %s <output_path> <archive_name> <xyz.PKT zyx.PKT ...>\r\n\n",PNAME);
   cputs("Where:\r\n\n");
   cputs("   <output_path>  = The path that the packets should be\r\n");
   cputs("                    moved TO.\r\n");
   cputs("   <archive_name> = Should be inserted here by your tosser.\r\n");
   cputs("                    This field is IGNORED!\r\n");
   cputs("   <xyz.PKT ...>  = The names of the packet files to move.\r\n\n");
   cputs("NOTE: Use in place of an archiver in your tosser, for the pointgate\r\n");
   cputs("      systems. This way, your tosser will think it's building an\r\n");
   cputs("      outbound archive, when in reality, it will just be moving the\r\n");
   cputs("      raw packets to a place where BlackHole can process them\r\n");
   cputs("      locally.\r\n\n");
   exit(1);
}   

int main(int argc, char * argv[])
{
   char iname[MAXPATH],
        oname[MAXPATH],
        dsk[MAXDRIVE],
        dir[MAXDIR],
        nam[MAXFILE],
        ext[MAXEXT],
        cdsk[MAXDRIVE],
        cdir[MAXDIR],
        cwd[MAXDRIVE + MAXDIR - 1];

   FlexList Plist(FLstrings);

   struct ffblk sr;

   textattr(LIGHTCYAN);
   clrscr();

#ifdef __BETA__
   cprintf("%s v.%u.%02u\r\n",PNAME,PVERS >> 8,PVERS & 0x00FF);
#else
   cprintf("%s v.%u.%02u.%1X\r\n",PNAME,PVERS >> 8,PVERS & 0x00FF,BVERS);
#endif
   cprintf("Copyright (C) %s, by %s\r\n\n",PCOPY,PAUTH);

   textattr(CYAN);
   
   Plist.setCompare(FLcomparE(stricmp));
   getcwd(cwd,MAXDRIVE + MAXDIR - 1);
   if (cwd[strlen(cwd) - 1] != '\\')
      strcat(cwd,"\\");
   fnsplit(cwd,cdsk,cdir,NULL,NULL);

   if (argc < 4)
      usage();

   for (int i = 1; i < argc; i++)
   {
      if ((argv[i][0] == '?') || ((argv[i][0] == '/') && (argv[i][0] == '?')))
         usage();
      strupr(argv[i]);
      switch (i)
      {
         case 1:
            strncpy(oname,argv[i],MAXDRIVE + MAXDIR - 1);
            if (oname[strlen(oname) - 1] != '\\')
               strcat(oname,"\\");
            break;
         case 2:
            break;
         default:
            if (fnsplit(argv[i],dsk,dir,nam,ext) & (DIRECTORY | DRIVE))
               Plist.insSortD(argv[i]);
            else
            {
               fnmerge(iname,cdsk,cdir,nam,ext);
               Plist.insSortD(iname);
            }
            break;
      }
   }

   fnsplit(oname,dsk,dir,NULL,NULL);

   Plist.mkcur();
   while (Plist.nextD(iname) != NULL)
   {
      fnsplit(iname,NULL,NULL,nam,ext);
      fnmerge(oname,dsk,dir,nam,ext);
      cprintf("Moving %s => %s\r\n",iname,oname);
      movepkt(oname,iname);
   }
   
   return (0);
}
